#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#define NSNPS 4
#define NSAMPLES 83 
#define BUFFSIZE 6300

int iter;
int flag;

#define REF 0
#define VAR 1

#define DEBUG 0

void simpheno(int *G, int ng, double R, int *P);
void shuffle(int *A, int n);
double checkOddsRatio(int *G, int ng, int *P);
void checkData(int *G, int *P, int ng);


int main(int argc, char **argv)  {


  FILE *fp, *fq; 

  const char *fgeno = "./tiny.geno";
  const char *fpheno = "./tiny.pheno";
  char fname[100];
  char buff[BUFFSIZE];
  int G[BUFFSIZE];
  int P[BUFFSIZE];

  int i = atoi(argv[1]);
  double R = 1.0 + i*0.1;
  int j;
  int count = 0;

  srand(time(NULL));

  if ( (fp = fopen(fgeno,"r")) == NULL )  {
    printf("*** error:  can't open %s ***\n", fgeno);
    exit(1);
  }
  sprintf(fname, "%s.%d", fpheno, i); 
  fq = fopen(fname,"w");

  for(j=0;j<NSNPS;j++)  {          

    int k;
    fgets(buff, BUFFSIZE, fp); 
    for(k=0;k<NSAMPLES;k++)  {
      G[k] = (int) (buff[k] - '0');
    }

    iter = j;
    simpheno(G, NSAMPLES, R, P);

    for(k=0;k<NSAMPLES;k++)  {
      fprintf(fq, "%1d", P[k]);
    }
    fprintf(fq, "\n");

    printf(" R = %f\n", checkOddsRatio(G,NSAMPLES,P));
    checkData(G, P, NSAMPLES);
    printf("\n");

  }

  fclose(fp);
  fclose(fq);

}

void checkData(int *G, int *P, int ng)  {
  int i, n0 = 0, n1 = 0, n2 = 0, m0 = 0, m1 = 0, m2 = 0;
  for(i=0;i<ng;i++)  {
    if ( P[i] == 1 && G[i] == 0 )
      n0++;
    else if ( P[i] == 1 && G[i] == 1 )
      n1++;
    else if ( P[i] == 1 && G[i] == 2 )
      n2++;
    else if ( P[i] == 0 && G[i] == 0 )
      m0++;
    else if ( P[i] == 0 && G[i] == 1 )
      m1++;
    else if ( P[i] == 0 && G[i] == 2 )
      m2++;
  }
  printf("Cases:     (%d,%d,%d)\n", n0, n1, n2);
  printf("Controls:  (%d,%d,%d)\n", m0, m1, m2);

}


double checkOddsRatio(int *G, int ng, int *P)  {
  int i;
  int icase = 0, ncase = 0, icontrol = 0, ncontrol = 0;
  double pcase, pcontrol, R1;

  for(i=0;i<ng;i++)  {
    if ( P[i] == 1 )  {
      ncase += 2;
      if ( flag == REF ) 
        icase += G[i];
      else
        icase += (2-G[i]);
    }

    else if ( P[i] == 0 )  {
      ncontrol += 2;
      if ( flag == REF ) 
        icontrol += G[i];
      else
        icontrol += (2-G[i]);
    }
  }

  pcase = (double) icase / (double) ncase;
  pcontrol = (double) icontrol / (double) ncontrol;

  R1 = ( pcase*(1.0-pcontrol) )/( pcontrol*(1.0-pcase) );
  printf("pcase = %10.4f  pcontrol = %10.4f  ", pcase, pcontrol);

  return R1;

}


void simpheno(int *G, int ng, double R, int *P)  {

  int i, j, N = 20; 
  int n0, n1, n2;
  int z0, z1, z2;
  double p0, p1, p2;

  /* Toggle ref/var (most CARe ref alleles are major alleles, so
     we choose one at random so the risk allele is the minor
     allele about half the time */

  /* This means a file needs to be generated which stores
     the ref/var risk allele.  Otherwise, the score-computing
     routine does not know which allele to count for, e.g., 
     EURA and which for EURC */

  if ( (double) rand() / RAND_MAX > 0.5 )  {
    flag = REF;
    z0 = 0;  z1 = 1;  z2 = 2;
  }
  else  {
    flag = VAR;
    z0 = 2;  z1 = 1;  z2 = 0;
  }

  /* Step 1 : choose 1000 controls at random */
  for(i=0;i<ng;i++)  {
    P[i] = 9;
  }

  j = 0;
  while ( j < N )  {
    int ir = rand() % ng;
    if ( P[ir] != 9 || G[ir] == 9 ) {
      continue;
    }
    else  {
      P[ir] = 0;  
      j++;
    }
  }

  /* Step 2 : choose N cases */
  n0 = n1 = n2 = 0;
  for(i=0;i<ng;i++)  {
    if ( G[i] == 9 ) 
      continue;
    else if ( G[i] == z0 )
      n0++;
    else if ( G[i] == z1 )
      n1++;
    else if ( G[i] == z2 )
      n2++;
  }


  p0 = 1/(n0 + R*n1 + R*R*n2); 
  p1 = R/(n0 + R*n1 + R*R*n2); 
  p2 = R*R/(n0 + R*n1 + R*R*n2); 
  
  j = 0;
  while ( j < N )  {
    int ir = rand() % ng;
    if ( P[ir] != 9 || G[ir] == 9 ) {
      continue;
    }
    else  {
      double r = (double) rand() / RAND_MAX;
      if ( G[ir] == z0 )  {
        if ( r < p0 )  {
	  P[ir] = 1;
	  j++;
	}
      }
      else if ( G[ir] == z1 )  {
        if ( r < p1 )  {
	  P[ir] = 1;
	  j++;
	}
      }
      else if ( G[ir] == z2 )  {
        if ( r < p2 )  {
	  P[ir] = 1;
	  j++;
	}
      } 
    } 
  }
  return;

} 

void shuffle(int *A, int n)  {
  int i;
  for(i=0;i<n;i++)  {

    int j = (int) ( n * (double) rand() / RAND_MAX );
    if ( j == n ) j--;
    int temp = A[i];
    A[i] = A[j];
    A[j] = temp;
  }
}

