#ifndef _SCOREADMGC_
#define _SCOREADMGC_

#include <iostream>
#include <fstream>
#include <cmath>

#include "Exception.h"
#include "IOHandler.h"

using namespace std;


class ScoreSUM;

class ScoreADMGC  {

  public:

    friend class ScoreSUM;

    // constructor
    ScoreADMGC(char *paramfile);

    // destructor
    ~ScoreADMGC(void);

    // main loop
    int evaluate(void);
    void computeMeanSdGamma();
    void computeMeanSdGamma(string ancfilename);

  private:

    // constructor for ScoreSUM
    ScoreADMGC(int nsmp, int nsnp, int *p, int *a, double *t, string ancfilename);
    int do_not_deallocate;


    int nsamples;
    int nsnps;

    ifstream genoStream;        // unused for now:  only for computing odds ratio
    ifstream phenoStream;
    ifstream localAncStream;
    ofstream outputStream;
    ofstream oddsratioStream;

    int *dipGenoVec;            // unused for now:  only for computing odds ratio
    int *dipPhenoVec;
    int *dipLocalAnc;

    double *globalAnc;

    double sdGamma;
    double meanGamma;

    // Note:  oddsratios require genotype and integral ancestry.
    string oddsratio1, oddsratio2;
    void setOddsRatios(double radmix);

    IOHandler ioh;

    double evalA1();

    int *AS;
    double *TS;

    int ncase;
    double nae, nee;

    void initializeDip(void);
    void finalizeDip(void);

    double f(double r);
    double df(double r);
    double ddf(double r);
    double find_initial_rval(void);
    double findroot(double, int *);

};

#endif
