#ifndef _SCORES1S_
#define _SCORES1S_

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <cmath>

#include "IOHandler.h"
#include "Exception.h"

using namespace std;

class ScoreS1S  {

  public:

    // constructor
    ScoreS1S(char *paramfile);

    // destructor
    ~ScoreS1S(void);

    // main loop
    int evaluate(void);


  private:

    int nsamples;
    int nsnps;
    int errflag;  // remove when exception handling introduced

    bool geno_is_dosage;

    ifstream genoStream;
    ifstream phenoStream;
    ifstream localAncStream;

    ofstream outputStream;
    ofstream oddsratioStream;

    int    *dipGenoVec;
    int    *dipPhenoVec;
    int    *dipLocalAnc;

    double* genoDosages;

    IOHandler ioh;

    double evalS1S();

    // data
    double XAARRT, XAARVT, XAAVVT, XAARRS, XAARVS, XAAVVS;
    double XAERRT, XAERVT, XAEVVT, XAERRS, XAERVS, XAEVVS;
    double XEERRT, XEERVT, XEEVVT, XEERRS, XEERVS, XEEVVS;

    double XART,XAVT,XARS,XAVS;
    double XERT,XEVT,XERS,XEVS;

    string oddsratio1, oddsratio2;   
    void setOddsRatios(double pAT, double pAS, double pET, double pES);

    // overloaded parameter evaluators 
    void countAll(int *G, int *P, int *A);

    // print parameters for exception message
    string errorString(void);

    inline double initpA(void);
    inline double initpE(void);
    inline double initpAT(void);
    inline double initpET(void);
    inline double initpAS(void);
    inline double initpES(void);

    double rootpA(double, double, double, double, double, double);
    double rootpE(double, double, double, double, double, double);
    double S(double, double, double, double, double, double, double);
    double d2S(double, double, double, double, double);

};

#endif
