#ifndef _SCORES4S_
#define _SCORES4S_

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <cmath>

#include <gsl/gsl_errno.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_min.h>

#include "IOHandler.h"
#include "Exception.h"

using namespace std;

class ScoreSUM;

struct s4sparams  {
  double XAARRT, XAARVT, XAAVVT, XAARRS, XAARVS, XAAVVS;
  double XAERRT, XAERVT, XAEVVT, XAERRS, XAERVS, XAEVVS;
  double XEERRT, XEERVT, XEEVVT, XEERRS, XEERVS, XEEVVS;
 //expected allele counts in cases/controls from dosages
  double XART,XAVT,XARS,XAVS;
  double XERT,XEVT,XERS,XEVS;
  double rhatA, rhatE;
  double impPa,impPe;
  double R, pA, pE;
};
typedef struct s4sparams s4sparams_t;


class ScoreS4S  {

  public:

    friend class ScoreSUM;

    // constructor
    ScoreS4S(char *paramfile);

    // destructor
    ~ScoreS4S(void);

    // main loop
    int evaluate(void);


  private:

    // constructors for ScoreSUM
    ScoreS4S(int n, int *g, int *p, int *a);
    ScoreS4S(int n, double *d, int *p, int *a);
    ScoreS4S(int n, int *g, int *p, double *a0, double *a1, double *a2);
    // problem:  the destructor is going to deallocate the arrays
    // kludge:  set flag in constructor do_not_deallocate
    bool do_not_deallocate;
    bool use_adjustment;
    bool geno_is_dosage;

    int nsamples;
    int nsnps;

    ifstream genoStream;
    ifstream phenoStream;
    ifstream localAncStream;

    ofstream outputStream;
    ofstream oddsratioStream;

    int    *dipGenoVec;
    int    *dipPhenoVec;
    int    *dipLocalAnc;

    string genoform;
    double *genoDosages;

    IOHandler ioh;

    double evalS4S();

    string oddsratio;
    void setOddsRatio(double r);

    // overloaded parameter evaluators 

    void init_s4sparams(double *D, int *P, int *A);
    void init_s4sparams(int *G, int *P, int *A);

    // backup for when Brent's algorithm fails
    int binsearch(double(*f)(double, s4sparams_t *), s4sparams_t *p, 
      double xmin, double xmax, double *xout);

    // print parameters for exception message
    string errorString(void);

    s4sparams_t *mys4sparams;

};

#endif
