#include "ScoreSUM.h"

ScoreSUM::ScoreSUM(char *paramfile)  {
  string str;
  ostringstream errstream;
  errstream << "[ScoreSUM::ScoreSUM]  ";
 
  ioh.readParams(paramfile);

  nsamples = ioh.getPositiveIntParam("nsamples");
  nsnps = ioh.getPositiveIntParam("nsnps");

  ioh.openInput("genofile", &genoStream);
  dipGenoVec = new int[nsamples];

  ioh.openInput("phenofile", &phenoStream);
  dipPhenoVec = new int[nsamples];
  globalAnc = new double[nsamples];

  genoDosages = new double[nsamples];
  // geno_is_dosage = ioh.getBoolParam("dosages");
  use_empirical_variance = ioh.getBoolParam("useempiricalvariance");
  ioh.getParam("genoformat",&genoform);

  ioh.openInput("ancfile",&localAncStream);
  dipLocalAnc = new int[nsamples];

  if ( ioh.getBoolParam("output2file") )  {
    ioh.openOutput("outfile",&outputStream);
  }
}

ScoreSUM::~ScoreSUM(void)  {

  if ( outputStream.is_open() )       {  outputStream.close();  }
  if ( genoStream.is_open() )         {  genoStream.close();    }
  if ( phenoStream.is_open() )        {  phenoStream.close();   }
  if ( localAncStream.is_open() )     {  localAncStream.close();   }

  delete dipGenoVec;
  delete dipPhenoVec;
  delete dipLocalAnc;
}


int ScoreSUM::evaluate(void)  {
  string str;

  ostringstream errstream;
  errstream << "[ScoreSUM::evaluate]  ";

  bool output_to_file = ioh.getBoolParam("output2file");

  try {  ioh.readIntLine(&phenoStream, nsamples, &dipPhenoVec);  }
  catch ( Exception& e )  {
    errstream << endl << e.getMessage();
    throw( Exception(errstream.str()) );
  }

  ioh.getParam("thetafile", &str);
  try  {  ioh.readOneColumnFile(str, nsamples, &globalAnc);  }
  catch ( Exception& e )  {
    errstream << "error reading blobalanc" << endl;
    errstream << e.getMessage();
    throw( Exception(errstream.str()) );
  }


  // for(int snpcount=0;snpcount<nsnps;snpcount++)  {
  for(int snpcount=0;snpcount<nsnps;snpcount++)  {

    if( !genoform.compare("dosage") ){
      geno_is_dosage = true;
      try { ioh.readRealLine(&genoStream, nsamples, &genoDosages); }
      catch ( Exception& e )  {
        errstream << "SNP # " << snpcount << endl;
        errstream << "error reading genotype" << endl;
        errstream << e.getMessage();
        throw( Exception(errstream.str()) );
      }
      for(int i=0;i<nsamples;i++)  {      // need to fill int here for S4S
        double g = genoDosages[i];
        dipGenoVec[i] = (int) (g + 0.5);  // round
      }
    }
    else if ( !genoform.compare("prob") )  {
      geno_is_dosage = true;
      double *tempVec = new double[2*nsamples];
      try { ioh.readRealLine(&genoStream, 2*nsamples, &tempVec); }
      catch ( Exception& e )  {
        errstream << "SNP # " << snpcount << endl;
        errstream << "error reading genotype" << endl;
        errstream << e.getMessage();
        throw( Exception(errstream.str()) );
      }
      for(int i=0;i<nsamples;i++)  {
        double g = 2.0*tempVec[2*i] + tempVec[i];
        dipGenoVec[i] = (int) (g + 0.5);  // round
      }
      delete tempVec;
    }
    else  {    // genoformat is "integer"
      geno_is_dosage = false;
      try { ioh.readIntLine(&genoStream, nsamples, &dipGenoVec); }
      catch ( Exception& e )  {
        errstream << "SNP # " << snpcount << endl;
        errstream << "error reading genotype" << endl;
        errstream << e.getMessage();
        throw( Exception(errstream.str()) );
      }
    }

    try { ioh.readIntLine(&localAncStream, nsamples, &dipLocalAnc); }  
    catch ( Exception& e ) {
      errstream << "SNP # " << snpcount << endl;
      errstream << "error reading diploid localanc" << endl;
      errstream << e.getMessage();
      throw( Exception(errstream.str()) );
    }

    double s4s, a1;


    try  {
      ScoreS4S S4S(nsamples, dipGenoVec, dipPhenoVec, dipLocalAnc);
      s4s = S4S.evalS4S();
    }
    catch ( Exception& e )  {
      errstream << "SNP # " << snpcount << endl;
      errstream << "error computing score S1S" << endl;
      errstream << e.getMessage();

      cerr << errstream.str();
      if ( output_to_file )  {
        outputStream << "no-score" << endl;
      }
      continue;
    }

    try  {
      if ( use_empirical_variance )  {

        string ancfilename;
        ioh.getParam("ancfile",&ancfilename);

        ScoreADMGC A1(nsamples, nsnps, dipPhenoVec, dipLocalAnc, globalAnc, ancfilename);
        a1 = A1.evalA1();

      }
      else  {
        ScoreA1 A1(nsamples, dipPhenoVec, dipLocalAnc, globalAnc);
        a1 = A1.evalA1();
      }
    }
    catch ( Exception& e )  {
      errstream << "SNP # " << snpcount << endl;
      errstream << "error computing score A1" << endl;
      errstream << e.getMessage();

      cerr << errstream.str();
      if ( output_to_file )  {
        outputStream << "no-score" << endl;
      }
      continue;
    }
 
    if ( output_to_file )  {
      outputStream << fixed << setw(10) << setprecision(4) << (s4s+a1) << endl;
    }
    else  {
      cout << fixed << setw(10) << setprecision(4) << (s4s+a1) << endl;
    }
  }
  return 0;
}













